<?php
namespace Neuron\Person\Attribute\Storage;

use Neuron\Db\Storage\Pager;
use Neuron\Person\Attribute;
use Neuron\Person\Attribute\Storage;
use Zend\Db\Adapter\Adapter;
use Zend\Stdlib\ArrayUtils;
use Throwable;

class Mysql extends \Neuron\Db\Storage implements Skeleton
{
    public function load($id, $mode = Storage::LOADBY_ID){
        try {
            $select = $this->select()
                ->from($this->_tables['person_attributes']);

            switch ($mode) {
                case Storage::LOADBY_ID:
                    $select->where($this->__(['person_attribute_id' => $id]));
                    break;
                case Storage::LOADBY_PERSONID_ATTR_CODE:
                    $select->where($this->__(['person_id' => $id['person_id']]));
                    $select->where($this->__(['attr_code' => $id['attr_code']]));
                    break;
                default:
                    $select->where($this->__(['person_attribute_id' => $id]));
            }
            return $this->fetchRow($select);
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function save(Attribute $Attribute, $update = false){
        try {
            /* mode update? */
            if ($update === true) {
                $update = $this->update()
                    ->table($this->_tables['person_attributes'])
                    ->set($this->__($Attribute->pull()))
                    ->where($this->__(['person_attribute_id' => $Attribute->person_attribute_id]));
                
                if ($this->execute($update)) {
                    /* update sukses, return Attribute id */
                    return $Attribute->person_attribute_id;
                }
            } else {
                $insert = $this->insert()
                    ->into($this->_tables['person_attributes'])
                    ->autoincrement($this->_('person_attribute_id'))
                    ->values($this->__($Attribute->pull()));

                if ($id = $this->execute($insert)) {
                    /* insert sukses, return Attribute id */
                    return $id;
                }
            }

            /* default gagal */
            return false;
        } catch (Throwable $throwable) {
            error_log('Error at ' . $throwable->getFile() . ' line ' . $throwable->getLine() . ': ' . $throwable->getMessage());
            return false;
        }
    }

    public function loadList($ids, $mode = Storage::LIST_ALL, $params = null){
        /* base select */
        $select = $this->select()
            ->from(['a' => $this->_tables['person_attributes']]);
        /* loadby page */
        if ($mode === Storage::LISTBY_PAGE) {
            /* get pager */
            $pager = Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('attribute_id', 'a.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('name', 'a.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('description', 'a.'), '%' . trim($ids) . '%')
                    ->unnest();
            }
        } else {
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case Storage::LISTBY_PERSONID:
                    $select->where([$this->_('person_id', 'a.') => $ids]);
                    break;
				case Storage::LOADBY_ATTR_CODE:
                    $select->where([$this->_('attr_code', 'a.') => $ids]);
                    break;
            }
        }

        try {
            //echo @str_replace('"','', $select->getSqlString());die();
            $rows = $this->fetchAll($select);
            $result = new \stdClass();
            $result->list = $rows;
            $result->pager = $pager;
            $total = isset($rows->total) ? $rows->total : 0;
            $result->countTotal = $total;
            return $result;
        } catch (\Throwable $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
}
