<?php
namespace Neuron\Person\Controller;

use Neuron\Person\Educations;
use Neuron\Person\Education as EducationModel;
use Neuron\Generic\Result;

class Education extends \Neuron\Person\Controller {
	protected function getView($name, $variables = array()){
		/* default layout & view prefix */
		$layout = 'neuron/person/layout/form';
		$prefix = 'neuron/person/form';

		/* libraries */
		$this->using('jquery', null, false)
		->using('bootstrap', null)
		->using('font-awesome', null)
		->using('bootbox');

		/* get layout n view prefix from config */
		$config = $this->getConfig();
		if (isset($config['task'])) {
		$config = $config['task'];
		$layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
		$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}


		// Script FCM
			$config = $this->getConfig();
	        $opt = new Notification\FCM\Options();
	        $opt->push($config['notification']['fcm']);
	        
	        $template = new Notification\Template(Notification\Template\Storage::factory($this->getDb(), $this->getConfig()));
	        
	        $fcm = new Notification\FCM(
	            Notification\Email\Storage::factory($this->getDb(), $this->getConfig()),
	            $template,
	            $opt
	        );
	        
	        $id_user = $this->userID();
	        $script = $fcm->listen('firebaseHandler', $id_user, $this->getPublicUrl());
			
		$variables['script'] = $script;

		/* set layout and view */
		$this->layout($layout);

		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

		return $view;
	}

	protected function getViewPartial($name, $variables = array()){
		$prefix = 'neuron/person/education';
		$config = $this->getConfig();
		if (isset($config['task'])) {
			$config = $config['task'];
			$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}

		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name)); // path to phtml file under view folder
		return $view;
	}

	// moved to /controller/widget/edu
	public function formAction()
	{
		$post = $this->getPostParams();
		$id = $post['id'];
		$person_id = $post['person_id'];
		$edudata = array();
		$title = "Create Education";

		$storage = \Neuron\Person\Parameter\Storage::factory($this->getDb(), $this->getConfig());
		$cpar = new \Neuron\Person\Parameters($storage);

		$post = $this->getPostParams();
		$arr = $cpar->listparam($post['param_ref_name']);
		$listEduType = array();
		$listEduLevel = array();
		if ($arr) {
			foreach ($arr as $data) {
				if($data['param_ref_name'] == 'EDUCATION_TYPE'){
					array_push($listEduType, $data);
				}else if($data['param_ref_name'] == 'EDUCATION_LEVEL'){
					array_push($listEduLevel, $data);
				}
			}
		}

		if($id){
			$storage = \Neuron\Person\Education\Storage::factory($this->getDb(), $this->getConfig());
			$cedu = new \Neuron\Person\Education($storage);
			$title = "Edit Education";
			if ($cedu->load($id))  {
				$edudata = $cedu->pull();
			}
		}

		return $this->getViewPartial('crud', get_defined_vars());
	}

	public function listAction() {
		$result = new Result();
        $session = $this->getSession()->get();
        $post = $this->getRest()->getParameters();


        $personId = $post['person_id'];
        try {
			$Educations = new Educations(EducationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $dataEducations = $Educations->load($personId, EducationModel\Storage::LISTBY_PERSON_TYPE);
			$dataEducation  = $dataEducations->data['list'];
			$dataFormal = [];
			$dataInformal = [];
			if($dataEducation){
				foreach ($dataEducation as $edu) {
					if($edu['education_type'] == 'FORMAL'){
						$dataFormal[] = $edu;
					}else{
						$dataInformal[] = $edu;
					}
				}
			}
            $result->data = ['formal'=> $dataFormal, 'informal'=> $dataInformal];
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed loading education");
        }

        $requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'API PERSON LIST EDUCATION';
		$logparam['identity'] = $personId ? $personId : '';
		$logparam['user'] = $session['user_id'] ? $session['user_id'] : '';
		$logparam['description'] = 'Log proses api person list education';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
	}

	public function loadAction() {
		$result = new Result();
        $session = $this->getSession()->get();
        $post = $this->getRest()->getParameters();


        $education_id = $post['education_id'];
        try {
            $education = new EducationModel(EducationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $education->load($education_id, EducationModel\Storage::LOADBY_ID);
            $result->data = $education->pull();
            
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed loading education");
        }

        $requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'API PERSON LOAD EDUCATION';
		$logparam['identity'] = $education_id ? $education_id : '';
		$logparam['user'] = $session['user_id'] ? $session['user_id'] : '';
		$logparam['description'] = 'Log proses person load education';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
	}

	public function saveAction() {
		$storage = \Neuron\Person\Education\Storage::factory($this->getDb(), $this->getConfig());
		
		$post = $this->getRest()->getParameters();
	
		
		try {
            $Education = new EducationModel(EducationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $result = $Education->create($post);
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed save Education");
        }

		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 

		$logparam['subject'] = 'PERSON SAVE EDUCATION';
		$logparam['identity'] = $this->getSession()->owner();;
		$logparam['data1'] = $result;
		$logparam['description'] = 'Log proses person save education';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($result);

		$this->writeLogAction($logparam);
		
		return $this->getRest()->getResponse($result);
	}

	public function deleteAction() {
		$result = new Result();
        $session = $this->getSession()->get();
        $post = $this->getRest()->getParameters();


        $education_id = $post['education_id'];
        try {
            $education = new EducationModel(EducationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $result = $education->delete($education_id);
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed loading relation");
        }

        $requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'API PERSON DELETE EDUCATION';
		$logparam['identity'] = $education_id ? $education_id : '';
		$logparam['user'] = $session['user_id'] ? $session['user_id'] : '';
		$logparam['description'] = 'Log proses person delete education';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);
	}

}
