<?php

namespace Neuron\Person\Education\Storage;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    protected $_nafManager = null;

    //fungsi lama
    public function listdata($params) {
        $select = $this->select()->from(array('edu' => $this->_tables['person_education']));
        
        if ((!empty($params['person_id']))) {
            $select->where(array('edu.person_id' => $params['person_id']));
        }
        if ((!empty($params['education_type']))) {
            $select->where(array('edu.education_type' => $params['education_type']));
        }
        
        if(!empty($params['order'][0]['column'])){
                $i = $params['order'][0]['column'];
                $dir = $params['order'][0]['dir'];
                $select->order(array($params['columns'][$i]['data'] . " " . $dir));
        }

        if(isset($params['start'])){
            $select->offset(intval($params['start']));
            $select->limit(intval($params['length']));
        }
        
        $data = $this->fetchAll($select);
        $dataAll = [];

        if($data) {
            foreach($data as $row) {
                array_push($dataAll, $row);
            }
        }

        $selectC = $this->select()->from(array('edu' => $this->_tables['person_education']));
        $selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
        if ((!empty($params['person_id']))) {
            $selectC->where(array('edu.person_id' => $params['person_id']));
        }
        if ((!empty($params['education_type']))) {
            $selectC->where(array('edu.education_type' => $params['education_type']));
        }
        $dataCount = $this->fetchRow($selectC);

        $total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
                        
        $result = ['data' => $dataAll, 'total' => $total];
        
        
        return $result;
    }

    public function load($perEduId) {
    $select = $this->select()
                    ->from($this->_tables['person_education'])
                    ->where(array('education_id' => $perEduId));

    return $this->fetchRow($select);
    }
	
	public function save(\Neuron\Person\Education $perEdu, $update = false) {
        if ($update) {
            if($perEdu->year_graduated == '') {$perEdu->year_graduated = null;}
            $update = $this->update()
                ->table($this->_tables['person_education'])
                ->set($this->__($perEdu->pull()))
                ->where($this->__(array('education_id' => $perEdu->education_id)));

            
            if ($this->execute($update)) {
                return $perEdu->education_id;
            }
          
        } else {
            if($perEdu->year_graduated == '') {$perEdu->year_graduated = null;}
            $insert = $this->insert()
                        ->into($this->_tables['person_education'])
                        ->autoincrement($this->_('education_id'))
                        ->values($this->__($perEdu->pull()));
            // echo str_replace('"','',$insert->getSqlString()).'<br />';  die();
            
            $save = $this->execute($insert);
            if ($save) {
                $perEdu->education_id = $save;
                return $perEdu->education_id;
            }
        }

        return false;
    }

    public function remove($perEduId) {
        $delete = $this->delete()
                        ->from($this->_tables['person_education'])
                        ->where(array('education_id' => $perEduId));
                    
        return $this->execute($delete);
    }

    public function loadList($ids, $mode = \Neuron\Person\Education\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
                ->from(array('a' => $this->_tables['person_education']));

        /* loadby page */
        if ($mode == \Neuron\Person\Education\Storage::LISTBY_PAGE) {
			if(isset($params['person_id']) && !empty($params['person_id'])){
				$select->where(array($this->_('person_id', 'a.') => $params['person_id']));
			}

			if(isset($params['education_type']) && !empty($params['education_type'])){
				$select->where(array($this->_('education_type', 'a.') => $params['education_type']));
			}

            if(isset($params['education_level']) && !empty($params['education_level'])){
				$select->where(array($this->_('education_level', 'a.') => $params['education_level']));
			}

            /* order */
            if(array_key_exists('order', $params) && array_key_exists('columns', $params)){
                foreach($params['order'] as $order){
                    $columnName = $params['columns'][$order['column']]['data'];
                    if($columnName == 'user_status_name'){
                        $colIdentity = $this->_('name', 'b.');
                    } else {
                        $colIdentity = $this->_($columnName, 'a.');
                    }
                    $select->order($colIdentity . ' ' . $order['dir']);
                }
            }
            
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('institution', 'd.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('departement', 'd.'), '%' . trim($ids) . '%')
                ->unnest();
            }

        } else {

            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case \Neuron\Person\Education\Storage::LISTBY_PERSON_TYPE:
                    $select->where(array($this->_('person_id', 'a.') => $ids));
                    break;
                case \Neuron\Person\Education\Storage::LISTBY_TYPE:
                    $select->where(array($this->_('education_type', 'a.') => $ids));
                    break;
                default:
            }

        }

        $list = is_array($ids) ? $ids : ['id' => $ids];
        $list = is_array($params) ? array_merge($list, $params) : $list;
        $this->triggerQueryPerson($select, $list, __FUNCTION__);

        // echo str_replace('"','',$select->getSqlString()); die();
        try {
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    private function triggerQueryPerson($query, $params, $caller) {

        /* trigger before query */
        if (($this->_nafManager === null) && defined('NAF_CAPABILITY_ADDON')) {
            if ($this->_nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
                $this->_nafManager->events()->trigger('person_education_query', $query, $params, $caller);
            }
        } else if(defined('NAF_CAPABILITY_ADDON')){
            $this->_nafManager->events()->trigger('person_education_query', $query, $params, $caller);
        }
    }

}