<?php

namespace Neuron\Person\Entity\Storage;

use Neuron\Db\Storage\Pager;
use Zend\Db\Adapter\Adapter;
use Zend\Db\Sql\Where;
use Zend\Db\Sql\Predicate\Expression;

use Neuron\Person\Entity\Storage;


class Mysql extends \Neuron\Db\Storage implements Skeleton {

	public function getTotalRows() {
		$select = $this->select()
										->columns(array('total' => $this->expression('FOUND_ROWS()')));
													 
		return $this->fetchValue($select);                
	}  

	public function listPerson($params) {
		$select = $this->select()
                    ->from(array('p' => $this->_tables['person']));
                    // ->where(array('(termination_date is null or termination_date > now())','p.status_id = 1'));

		$searchable = false;

		if (class_exists('Neuron\Employee\Controller\Employee') && class_exists('Neuron\Organization\Controller\Organization')){
			
			if(isset($params['filter_org_id']) && !empty($params['filter_org_id'])) {
					$select->join(array('ej' => $this->_tables['employee_jobs']), 'ej.employee_id = p.id', array('ohp_id'));
					$select->join(array('ohp' => $this->_tables['org_has_position']), 'ohp.id = ej.ohp_id', array('org_id' => 'organization_id'));
					$select->where(array('ohp.organization_id'=> $params['filter_org_id']));
					$select->where->nest()
					->greaterThan($this->_('end_date','ej.'),date("Y-m-d"))->or
					->isNull($this->_('end_date','ej.'))->unnest();

			}
		}

		if(isset($params['arr_ref_id'])) {
			$select->where(new Expression('p.id IN('.implode(',',$params['arr_ref_id']).')'));
		}

		//get by id
		if(!empty($params['id'])){
			$select->where(array('p.id'=> $params['id']));
		}

		if(isset($params['draw'])){
			if(isset($params['columns'])) {
				$arrColumn = [];
				
				foreach ($params['columns'] as $col) {
					if($col['data'] != null) {
						$arrColumn[$col['data']] = $col['name'];
					}
				}
	
				if(!empty($params['search']['value'])) {
					$searchable = true;
					$nest = $select->where->nest();

					foreach ($params['columns'] as $col) {
						if($col['data'] != null) {
							if($col['searchable'] === "true") {
								$nest->like(new Expression('LOWER('.$col['name'].')'),new Expression('\'%'.$params['search']['value'].'%\''));
								$nest->or;
							}		
						}
					}

					$nest->like(new Expression("LOWER( CONCAT( p.first_name, ' ',p.last_name ) )"), new Expression('\'%'.$params['search']['value'].'%\''));
	
					$nest->unnest();
					
				}
	
				$select->columns($arrColumn,false);
			}
			
			$i = $params['order'][0]['column'];
			$dir = $params['order'][0]['dir'];
			$select->order(array($params['columns'][$i]['data'] . " " . $dir));

			$start = isset($params->start) ? $params->start : $params['start'];
			$length = isset($params->length) ? $params->length : $params['length'];

			$offset = ($start/$length) * $length;
			$select->limit(intval($length));
			$select->offset(intval($offset));
		}

		$data = $this->fetchAll($select);

	    $dataAll = [];

        if($data) {
            foreach($data as $row) {
				if(!file_exists(getcwd() .'/public'. $row['pic_official'])){
					$row['pic_official'] = '/asset/images/upload/avatar/user.png';
				} 
                array_push($dataAll, $row);
            }
		}	

		// reset join column handling error saat count mysql 5.7.29
		$selectC = $select;
		$joins = $selectC->joins->getJoins();
		$selectC->reset(\Zend\Db\Sql\Select::JOINS);
		foreach ($joins as $key => $join) {
			//re add join without cols
			$join['columns'] = [];
			$type = $join["type"];
			$selectC->join($join["name"], $join["on"], [], $type);
		}
		$selectC->reset('limit')->reset('offset');
		$selectC->reset(\Zend\Db\Sql\Select::ORDER);
        $selectC->columns(array('total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')));
        if(isset($params['ref_id'])){
            $selectC->where(array('t.ref_id' => $params['ref_id']));
        }
		// echo('$select=> '.str_replace('"','',$selectC->getSqlString()));die();
		$dataCount = $this->fetchRow($selectC);
		
		$countTotal = 0;
        
		if(isset($dataCount['total_count'])){
			$countTotal = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
		}
        $filtered = $countTotal;
		
		$result = ['data' => $dataAll, 'total' => $countTotal, 'filtered' => $filtered];
		
        return $result;
	}
	
	public function save(\Neuron\Person\Entity $person, $update = false) {
		if ($update == true) {
			$update = $this->update()
				->table($this->_tables['person'])
				->set($this->__($person->pull()))
				->where($this->__(array('id' => $person->id)));
			
			if ($this->execute($update)) {
				return $person->id;
			}
			
		} else {
			$insert = $this->insert()
			->into($this->_tables['person'])
			->values($this->__($person->pull()));
			
			$save = $this->execute($insert);
			if ($save) {
				$person->id = $save;
				return $person->id;
			}
		}

		return false;
	}
	
	public function load($id, $mode = Storage::LOADBY_ID) {
		$select = $this->select()
			->from($this->_tables['person']);

		switch ($mode){
			case Storage::LOADBY_ID :
				$select->where($this->__(array('id' => $id)));
				break;
			case Storage::LOADBY_CODE :
				$select->where($this->__(array('code' => $id)));
				break;
		}
		try {
			return $this->fetchRow($select);
		} catch (\Throwable $ex) {
			error_log('Error occured from :'.$ex->getFile().' Line : '.$ex->getLine().' :: '.$ex->getMessage());
			return false;
		}
	}

	public function remove($id) {
		$delete = $this->delete()
						->from($this->_tables['person'])
						->where(array('id' => $id));
		return $this->execute($delete);
	}

	public function loadPerson($id, $field){
		try {
			$select = $this->select()
											->from($this->_tables['person'])
											->where(array($field => $id));
			// echo str_replace('"','',$select->getSqlString()).'<br />';
			return $this->fetchAll($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function saveMapping($data, $update = false){
		try {
			if ($update == true) {
				$update = $this->update()
					->table($this->_tables['person_user_map'])
					->set($this->__(array('person_id'=>$data['personId'],
											'user_id'=>$data['userId'],
											'start_date'=>$data['startDate'],
											'end_date'=>$data['endDate'])))
					->where($this->__(array('user_map_id' => $data['mappingId'])));

				// echo str_replace('"','',$update->getSqlString()).'<br />';
				if ($this->execute($update)) {
						return $data['mappingId'];
				}
			} else {
				$insert = $this->insert()
					->into($this->_tables['person_user_map'])
					->values($this->__(array('person_id'=>$data['personId'],
											'user_id'=>$data['userId'],
											'start_date'=>$data['startDate'],
											'end_date'=>$data['endDate'])));
				$save = $this->execute($insert);
				if ($save) {
					return $save;
				}
			}
		} catch (Exception $e) {
			return false;
		}

	}

	public function loadMapping($personId){
		try {
			$select = $this->select()
						->from(array('pum'=>$this->_tables['person_user_map']))
						->where(array('person_id' => $personId, '(end_date is null or end_date > now())'));
			// echo str_replace('"','',$select->getSqlString()).'<br />';
			return $this->fetchRow($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function checkMapping($personId, $userId){
		try {
			$select = $this->select()
							->from(array('pus'=>$this->_tables['person_user_map']))
							->where(array('user_id'=>$userId, '(end_date is null or end_date > now())', 'person_id !='.$personId));
			return $this->fetchRow($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function listMapping($params = array()){
		$select = $this->select()
		->from(array('pus'=> $this->_tables['person_user_map']))
		->join(array('p'=>$this->_tables['person']), 'p.id = pus.person_id', array('person_name'=>$this->expression("CONCAT(first_name, ' ', last_name)")),'left')
		->where(array('(end_date is null or end_date > now())'));

		if(isset($params['length'])){
			$select->limit(intval($params['length']));
			if(isset($params['offset'])){
				$select->offset(intval($params['start']));
			}
		}
		// echo str_replace('"','',$select->getSqlString()).'<br />';  
		
		try {
			return $this->fetchAll($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function loadMap($id, $field){
		try {
			$select = $this->select()
			->from($this->_tables['person_user_map'])
			->where(array($field => $id));

			return $this->fetchRow($select);
		} catch (Exception $e) {
			return false;
		}
	}

	public function checkPersonCode($code){
		try {
			$select = $this->select()
						->from($this->_tables['person'])
						->where(array('code'=>$code));
			return $this->fetchAll($select);
		} catch (Exception $e) {
			return false;
		}

	}

	public function loadPersonDetail($id){

		$select = $this->select()
						->quantifier($this->expression('SQL_CALC_FOUND_ROWS'))
						->from(array('p' => $this->_tables['person']))
						->join(array('pt' => $this->_tables['person_type']), 'pt.type_id = p.type_id', array('type_name' => 'name'))
						->join(array('ps' => $this->_tables['person_status']), 'ps.status_id = p.status_id', array('status_name' => 'description'))
						->where(array('p.id'=>$id));
						// echo str_replace('"','',$select->getSqlString()).'<br />';  
						
		return $this->fetchRow($select);
	}

	public function loadList($ids, $mode = Storage::LIST_ALL, $params = null){
        /* base select */
        $select = $this->select()->from(['p' => $this->_tables['person']]);
        /* loadby page */
        if ($mode === Storage::LISTBY_PAGE) {
            /* get pager */
            $pager = Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('first_name', 'p.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('last_name', 'p.'), '%' . trim($ids) . '%')
                    ->or
					->like(new Expression("CONCAT(TRIM(p.first_name),' ',TRIM(p.last_name))"), '%' . trim($ids) . '%')
                    ->unnest();
            }
        } else {
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case Storage::LISTBY_TYPE_ID:
                    $select->where([$this->_('type_id', 'p.') => $ids]);
                    break;
                case Storage::LISTBY_STATUS_ID:
                    $select->where([$this->_('status_id', 'p.') => $ids]);
                    break;
				case Storage::LISTBY_ID:
					if(is_array($ids)){
						$select->where->in($this->_('id','p.')  ,$ids);
					}else{
						$select->where([$this->_('id','p.') => $ids]);
					}
					break;
            }
			
			/* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('first_name', 'p.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('last_name', 'p.'), '%' . trim($ids) . '%')
					->or
					->like(new Expression("CONCAT(TRIM(p.first_name),' ',TRIM(p.last_name))"), '%' . trim($ids) . '%')
                    ->unnest();
            }
        }
		
		if($params['draw']){
			$i = $params['order'][0]['column'];
			$dir = $params['order'][0]['dir'];
			$select->order(array($params['columns'][$i]['data'] . " " . $dir));
			
			$start = isset($params->start) ? $params->start : $params['start'];
			$length = isset($params->length) ? $params->length : $params['length'];

			$offset = ($start/$length) * $length;
			$select->limit(intval($length));
			$select->offset(intval($offset));
		}
		
		//echo str_replace('"','',$select->getSqlString()).'<br />'; 

        try {
            $rows = $this->fetchAll($select);
            $result = new \stdClass();
            $result->rows = $rows;
            $result->pager = $pager;
            $total = isset($rows->total) ? $rows->total : 0;
            $result->countTotal = $total;
            return $result;
        } catch (\Throwable $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }
	
	public function countEmpGender(){
    	$select = $this	->select()
						->columns(array(
							'total' 		=> $this->expression('COUNT(*)'),
							))
						->from(['p' => $this->_tables['person']]);

		if (class_exists('Neuron\Employee\Controller\Employee')){
			$select->join(array('e' => $this->_tables['employee']), 'e.person_id = p.id', array());
		}

    	$totalMale 			= 0;
    	$totalFemale 		= 0;
    	$result 			= new \stdClass;

    	/*Fetch Male*/
		
		if (class_exists('Neuron\Employee\Controller\Employee')){
			$select->where(array('e.status_id' => 1));
		}
		
    	$select				->where(array('gender' => 'M', 'p.id NOT IN (SELECT person_id FROM person_relation WHERE end_date IS NULL)'));
    	$totalMale 			= $this->fetchRow($select)['total'];

    	/* Fetch Female */
    	$select->reset('where');
		
		if (class_exists('Neuron\Employee\Controller\Employee')){
			$select->where(array('e.status_id' => 1));
		}
		
    	$select				->where(array('gender' => 'F', 'p.id NOT IN (SELECT person_id FROM person_relation WHERE end_date IS NULL)'));
    	$totalFemale 		= $this->fetchRow($select)['total'];

    	$result->totalFemale 	= $totalFemale;
    	$result->totalMale 		= $totalMale;

    	return $result;
    }


}
